
// Section 5 Review, Question 1 Solution

// Create a game setting interface where you are able to select the mode and map of
// a game you are creating.  Certain maps should support three modes, "Deathmatch",
// "Capture the Flag", and "Team Battle".  Maps should only appear in a selection
// list if the map is able to be played in a mode.  When the user changes the mode,
// the map should be reset.  Create a "Submit" button that is diabled when a mode
// or map isn't selected.  Create a game datum that will handle the form (you should
// define /game/Topic(), and all verification functions inside the datum) and will
// store the mode and map.

var/const
  MODE_DEATHMATCH	= 1
  MODE_CTF			= 2
  MODE_TEAM			= 4

var/list
  maps = list()
  modes = list("Deathmatch"			= MODE_DEATHMATCH,
               "Capture the Flag"	= MODE_CTF,
               "Team Battle"		= MODE_TEAM)

world/New()
  ..()
  for(var/m in typesof(/map) - /map)
    maps += new m

game
  var
    mob/creator
    mode
    map/map

  New(mob/_creator)
    src.creator = _creator
    src.DisplaySettingsForm()

  Topic(href, href_list)
    ..()

    var/action = href_list["action"]
    var/value = href_list["value"]

    switch(action)
      if("modes")
        if(src.isValidMode(value))
          src.mode = value
          src.map = null // reset map
          src.DisplaySettingsForm()

      if("maps")
        var/map/m = locate(value)
        if(m && isValidMap(m))
          src.map = m
          src.DisplaySettingsForm()

      if("continue")
        if(canContinue())
          src.creator << browse(null, "window=gamesettings")
          src.creator << "The game mode is [src.mode] and the map is [src.map.name]."
          del(src)

  proc
    canContinue() // can the player finish selecting the map and mode?
      return src.map && isValidMap(src.map) && isValidMode(src.mode)

    isValidMap(map/m) // is this a valid map?  Only allow maps to be set when a mode supports it
      return src.mode && (m.supported_modes & modes[src.mode])

    isValidMode(mode) // is this a valid mode?
      return (mode in modes)

    DisplaySettingsForm()
      var/modes_options = ""
      var/maps_options = ""

      for(var/i in modes)
        modes_options += {"<option[src.mode == i ? " selected" : ""] value="[i]">[i]</option>"}

      if(mode)
        for(var/map/m in maps)
          if(isValidMap(m))
            maps_options += {"<option[src.map == m ? " selected" : ""] value="\ref[m]">[m.name]</option>"}

      var/html = \
      {"
      <html>
      <head>
        <title> Game Settings </title>
        <script language="javascript">
          function set(input) {
            window.location="byond://?src=\ref[src]&action="+input.name+"&value="+input.value;
          }
        </script>
        <style type="text/css"> select { width: 100%; } </style>
      </head>
      <body>
        <table width="100%" height="100%">
        <tr align="center">
          <td width="40%"><b>Modes</td>
          <td width="60%"><b>Supported Maps</td>
        </tr>
        <tr align="center">
          <td>
            <select name="modes" onchange="set(this);" size="5">
              [modes_options]
            </select>
          </td>
          <td>
            <select name="maps" onchange="set(this);" size="5">
              [maps_options]
            </select>
          </td>
        </tr>
        <tr align="center" valign="center">
          <td colspan="2">
            <button[!canContinue() ? " disabled" : ""]
             onclick="window.location='byond://?src=\ref[src]&action=continue'">
             Continue</button>
          </td>
        </tr>
      </body>
      </html>
      "}

      src.creator << browse(html, "window=gamesettings;size=350x200;can_close=0")

mob
  var
    game/game
  verb
    game_settings()
      if(src.game) return // the "when" check
      src.game = new /game (src)

map
  var
    name
    supported_modes

  multimap
    name = "Multi Map"
    supported_modes = MODE_DEATHMATCH | MODE_TEAM | MODE_CTF

  deathbowl
    name = "Death Bowl"
    supported_modes = MODE_DEATHMATCH

  frenzy
    name = "Frenzy"
    supported_modes = MODE_DEATHMATCH | MODE_TEAM

  redvsblue
    name = "Red VS Blue"
    supported_modes = MODE_TEAM

  invasion
    name = "Invasion"
    supported_modes = MODE_TEAM | MODE_CTF

  houses
    name = "House"
    supported_modes = MODE_CTF

