
// Section 6 Review, Question 1 Solution

// Write a timed lock mini-game that allows players to play a game where he or she tries to open
// a door.  A form should appear when the player attempts to open the door.  The mini-game should
// let the player press a Start button to start the timer.  During this time, he or she has 30
// seconds to guess a 3 digit number with each digit being able to be a number between 1 and 5.
// Three selection lists (one list per digit) should show the numbers.  A form should be used to
// submit the code.  If the code is correct within the time limit, then the player may advance
// through the door.  Otherwise, security gets the player and he or she loses.  The player should
// have a "Give Up" button that is only accessible if less than 20 seconds has elapsed.  When the
// player either wins, loses, or gives up, the form should close.  The amount of time left should
// be displayed and updated on the form.  You may use GenericForm.dm, TimedForm.dm and AlertForm.dm
// (located in the suppliment) for the definitions of the generic form and timer.

#include "GenericForm.dm"
#include "TimedForm.dm"
#include "AlertForm.dm"

var/const
  LOCK_TIME = 300
  BASE = 5

genericForm/timed/lockcode
  window_name = "Door Lock"
  window_params = "size=250x250&can_close=0&can_minimize=0"
  update = 1
  var
    obj/door/door
    started = 0
    digits[3]
    list/errors = list()

  New(_owner, _door)
    src.door = _door
    digits[1] = "1"
    digits[2] = "1"
    digits[3] = "1"
    ..()

  Topic(href, list/href_list)
    . = ..()
    if(.)
      var/action = href_list["action"]
      var/value = href_list["value"]
      switch(action)
        if("start")
          if(!started)
            started = 1
            spawn() StartTimer(LOCK_TIME)
        if("end")
          if((started && time_left >= 20) || !started)
            del(src)

        if("dig1")  if(started) digits[1] = value
        if("dig2")  if(started) digits[2] = value
        if("dig3")  if(started) digits[3] = value

  GeneratePage()
    var/text
    var/style = "select { width: 100% }"
    var/js = {"
    function action(action) {
      window.location="byond://?src=\ref[src]&action="+action;
    }
    function set(input) {
      window.location="byond://?src=\ref[src]&action="+input.name+"&value="+input.value;
    }
    "}

    var/list/diglist = new /list (3)
    for(var/i = 1; i <= 3; i++)
      diglist[i] = ""
      for(var/j = 1; j <= BASE; j++)
        diglist[i] += "<option[digits[i]=="[j]" ? " selected" : ""] value=\"[j]\">[j]</option>"

    text = {"
      <table width="100%" height="100%">
        <tr height="10%" align="center">
          <td width="50%"> <button onclick="action('start');">Start</button> </td>
          <td width="50%">
            <button[started && time_left < 10 ? " disabled" : ""]
            onclick="action('end');">[started ? "Give Up" : "End"]</button>
          </td>
        </tr>
        <tr height="10%">
          <td colspan="2" align="center"> [started ? "Time Left: [time_left / 10] second\s" : ""] </td>
        </tr>
        <tr height="60%"><td colspan="2">
          <form name="lock" action="byond://" method="get">
            <input type="hidden" name="src" value="\ref[src]" />
            <input type="hidden" name="form" value="lock" />
            <table width="100%" height="100%">
              <tr height="80%" valign="top">
                <td width="33%">
                  <select[!started ? " disabled" : ""] multple size="[BASE]" name="dig1" onchange="set(this);">[diglist[1]]</select>
                  <span class="error">[errors["dig1"]]</span>
                </td>
                <td width="34%">
                  <select[!started ? " disabled" : ""] multple size="[BASE]" name="dig2" onchange="set(this);">[diglist[2]]</select>
                  <span class="error">[errors["dig2"]]</span>
                </td>
                <td width="33%">
                  <select[!started ? " disabled" : ""] multple size="[BASE]" name="dig3" onchange="set(this);">[diglist[3]]</select>
                  <span class="error">[errors["dig3"]]</span>
                </td>
              </tr>
              <tr><td align="center"></td></tr>
              <tr height="20%" valign="bottom">
                <td colspan="3" align="center"> <input[!started ? " disabled" : ""] type="submit" value="Check Code" /> </td>
              </tr>
            </table>
        </td></tr>
      </table>
    "}

    UpdatePage(text, style, js)

  ProcessVariable(formname, name, value)
    if(formname == "lock")
      switch(name)
        if("dig1")
          src.digits[1] = value
          if(!door.isCorrectLockDigit(value, 1))
            return "Error"
        if("dig2")
          src.digits[2] = value
          if(!door.isCorrectLockDigit(value, 2))
            return "Error"
        if("dig3")
          src.digits[3] = value
          if(!door.isCorrectLockDigit(value, 3))
            return "Error"
    return null

  FormSubmitSuccess(formname)
    door.Unlock()
    var/message = "Congradulations, you have guessed the code within the time limit!  The door \
      has been unlocked.  The code was [door.code]."
    new /genericForm/alert (owner, message)
    del(src)

  FormSubmitError(formname, list/err)
    src.errors = err
    ..()

  TimeUp()
    var/message = "Uh oh, you couldn't figure out the code.  Security finds you and arrests you."
    new /genericForm/alert (owner, message)
    ..()

  TimeStarted()
    DisplayPage()

obj/door
  density = 1
  New()
    ..()
    code = "[rand(1, BASE)][rand(1, BASE)][rand(1, BASE)]"
  var
    code = ""
    opened = 0

  proc
    isCorrectLockDigit(num, digit)
      return copytext(code, digit, digit+1) == num

    EnterDoor(mob/M)
      if(src.opened)
        M << "You are free to enter the door."
      else
        if(M.form) return
        M.form = new /genericForm/timed/lockcode (M.client, src)

    Unlock()
      src.opened = 1
      src.density = 0

mob
  Move()
    if(src.form) return 0
    return ..()

  Bump(atom/A)
    if(istype(A, /obj/door))
      var/obj/door/D = A
      D.EnterDoor(src)