
// Section 5 - "Data Verification", Example 2
// The occupation-picking example limits occupations to be picked depending
// on the player's gender, which he or she picks in the same page

var/list
  occupations = list("Soldier", "Doctor", "Teacher", "Housewife", "Nurse")
  genders = list("male" = list("Soldier", "Doctor", "Teacher"),
                 "female" = list("Housewife", "Nurse", "Teacher"))

mob
  var/occupation

  Topic(href, list/href_list)
    if(src != usr) return // to verify the clicker of the link is supposed to click it
    ..()

    var/action = href_list["action"]
    var/value = href_list["value"]

    switch(action)
      if("gender")
        // verify gender before setting it
        if(isValidGender(value))
          src.gender = value
          src.occupation() // update the window to show the new occupations
      if("occupation")
        if(isValidOccupation(value))
          src.occupation = value
          src.occupation()
      if("submit")
        if(canSubmit()) // the "when" check for the whole form
          src << "You are a [src.gender] [src.occupation]."
          src << browse(null, "window=occupation")

  proc
    canSubmit()
      // the user can only press the submit button when he or she has a valid gender
      // and a valid class
      return isValidGender(src.gender) && isValidOccupation(src.occupation)

    isValidGender(value) // the user can only be a male or female
      return value == "male" || value == "female"

    isValidOccupation(value)
      // a valid occupation is defined in the occupation list, and as an association
      // in the gender list to determine which occupations a gender can choose
      return (value in occupations) && (value in genders[src.gender])

  verb
    occupation()
      var/list_occupations = ""
      for(var/i in occupations)
        if(isValidOccupation(i)) // the "what" check for occupations
          list_occupations += {"<input[src.occupation == i ? " checked" : ""] type="radio"
          onclick="set(this);" name="occupation" value="[i]" /> [i] <br />"}

      var/html = \
      {"
        <html>
        <head>
          <title> Occupation </title>
          <script language="javascript">
            function set(input) {
              window.location="byond://?src=\ref[src]&action="+input.name+"&value="+input.value;
            }
          </script>
        </head>
        <body>
          Gender: <br />
            <input[src.gender == "male" ? " checked" : ""] type="radio"
             onclick="set(this);" name="gender" value="male" /> Male <br />
            <input[src.gender == "female" ? " checked" : ""] type="radio"
             onclick="set(this);" name="gender" value="female" /> Female <br />
          <br />
          Occupation: <br />
            [list_occupations]
          <br />
          <button[!canSubmit() ? " disabled" : ""] onclick="window.location='byond://?src=\ref[src]&action=submit'">Submit</button>
        </body>
        </html>
      "}

      src << browse(html, "window=occupation;size=200x300")
