
// Section 5 - "Data Verification", Example 3
// A form that tells the user whenever any user-inputted data is sent incorrectly.

mob
  name = "John Doe"
  desc = "I'm just the average joe."
  var
    age = 20

  Topic(href, list/href_list)
    if(src != usr) return
    ..()

    if(href_list["form"]) // determine if the link is a form if the "form" param was sent
      var/list/data = href_list.Copy(href_list.Find("form") + 1) // split the 'src' parameter

      var/errors = ProcessForm(href_list["form"], data) // process data
      if(istype(errors, /list))
        // there are errors.  resend the form with the error list
        DisplayForm(errors)
      else
        // close the form when submitted without any problems
        src << browse(null, "window=settings")

  proc
    ProcessForm(formname, list/params)
      var/list/errors = list()
      for(var/item in params) // loop through each name=value item individually
        var/alert = ProcessVariable(formname, item, params[item])
        if(alert) errors[item] = alert
      if(!length(errors)) return null
      return errors

    ProcessVariable(formname, name, value)
      // use the form name in order to have different functionality for different forms
      if(formname == "settings")
        switch(name)
          if("name")
            // let the name be set, but do not let the form be submitted if an invalid value was typed
            // this lets the user be able to keep their old input and modify it
            src.name = value
            if(!isValidName(value))
              return "Your name must be between 3 and 25 characters long"
          if("age")
            value = text2num(value)
            src.age = value
            if(!isValidAge(value))
              return "Invalid age"
          if("desc")
            src.desc = value
            if(!isValidDesc(value))
              return "Your description is too long";

    // verification

    isValidName(text)
      return length(text) <= 25 && length(text) >= 3

    isValidAge(age)
      return age > 0 && age == round(age)

    isValidDesc(text)
      return length(text) <= 300

    DisplayForm(list/errors=list())
      var/html = \
      {"
        <html>
        <head>
          <title> Settings </title>
          <style type="text/css"> span.error { font-size: 8pt; color: #ff0000 } </style>
        </head>
        <body>
          <form name="settings" action="byond://" method="get">
            <input type="hidden" name="src" value="\ref[src]" />
            <input type="hidden" name="form" value="settings" />

            Name: <input type="text" name="name" value="[src.name]" size="25"/> <span class="error">[errors["name"]]</span><br />
            Age: <input type="text" name="age" value="[src.age]" size="3" /> <span class="error">[errors["age"]]</span> <br />
            Description: (up to 300 characters) <span class="error">[errors["desc"]]</span> <br />
            <textarea name="desc" rows="5" cols="30" size="300">[src.desc]</textarea> <br />

            <input type="submit" value="Submit" />
          </form>
        </body>
        </html>
      "}

      src << browse(html, "window=settings;size=500x300")

  verb
    settings()
      src.DisplayForm()