
// Section 6 - "Organization", Example 3
// Example 5.3 rewritten using a form datum.  Verification is handled in a similar way,
// just split up a bit more and with slight logic modifications.

infoForm
  var
    client/owner

  New(client/_owner)
    ..()
    src.owner = _owner
    src.DisplayForm()

  Del()
    owner << browse(null, "window=settings")
    ..()

  Topic(href, list/href_list)
    // usr is always a mob in datum/Topic(), so we must compare the owner
    // with its client to verify the 'who' check
    if(src.owner != usr.client) return
    ..()

    if(href_list["form"])
      var/list/data = href_list.Copy(href_list.Find("form") + 1)

      var/errors = ProcessForm(href_list["form"], data) // process data
      if(istype(errors, /list))
        DisplayForm(errors)
      else
        del(src)

  proc
    DisplayForm(list/errors=list())
      var/mob/M = owner.mob
      var/html = \
      {"
        <html>
        <head>
          <title> Settings </title>
          <style type="text/css"> span.error { font-size: 8pt; color: #ff0000 } </style>
        </head>
        <body>
          <form name="settings" action="byond://" method="get">
            <input type="hidden" name="src" value="\ref[src]" />
            <input type="hidden" name="form" value="settings" />

            Name: <input type="text" name="name" value="[M.name]" size="25"/> <span class="error">[errors["name"]]</span><br />
            Age: <input type="text" name="age" value="[M.age]" size="3" /> <span class="error">[errors["age"]]</span> <br />
            Description: (up to 300 characters) <span class="error">[errors["desc"]]</span> <br />
            <textarea name="desc" rows="5" cols="30" size="300">[M.desc]</textarea> <br />

            <input type="submit" value="Submit" />
          </form>
        </body>
        </html>
      "}

      owner << browse(html, "window=settings;size=500x300")

    ProcessForm(formname, list/params)
      var/list/errors = list()
      for(var/item in params) // loop through each name=value item individually
        var/alert = ProcessVariable(formname, item, params[item])
        if(alert) errors[item] = alert
      if(!length(errors)) return null
      return errors

    ProcessVariable(formname, name, value)
      var/mob/M = owner.mob
      // use the form name in order to have different functionality for different forms
      if(formname == "settings")
        switch(name)
          if("name")
            // let the name be set, but do not let the form be submitted if an invalid value was typed
            // this lets the user be able to keep their old input and modify it
            M.name = value
            if(!M.isValidName(value))
              return "Your name must be between 3 and 25 characters long"
          if("age")
            value = text2num(value)
            M.age = value
            if(!M.isValidAge(value))
              return "Invalid age"
          if("desc")
            M.desc = value
            if(!M.isValidDesc(value))
              return "Your description is too long";


mob
  name = "John Doe"
  desc = "I'm just the average joe."
  var
    infoForm/form
    age = 20

  proc
    isValidName(text)
      return length(text) <= 25 && length(text) >= 3

    isValidAge(age)
      return age > 0 && age == round(age)

    isValidDesc(text)
      return length(text) <= 300

  verb
    settings()
      if(src.form) return
      src.form = new /infoForm (src.client)