
// Section 6 - "Organization", Example 6
// Generic Form version 2.  Implements functions that can easily handle forms, and
// data verification.  Modifies UpdatePage() to allow custom CSS

// Generic Form version 2

mob/var
  form // the form the user is currently viewing

genericForm
  var
    client/owner // the viewer of the form

    window_name = "Generic Form" // name of title bar and browser window id
    window_params = "" // any other generic parameters sent through browse()
    body = "" // the html to be displayed to the viewer

  New(client/_owner)
    ..()
    src.owner = _owner
    DisplayPage() // display any page right when it's instantiated

  Del()
    owner << browse(null, "window=[src.window_name]")
    ..()

  Topic(href, list/href_list)
    // When overridden, you must check if the parent implementation returns true
    // in order to ensure the code isn't run when there is an invalid Topic() call
    // EG: . = ..(); if(.) { ... }

    if(!isValidViewer(usr.client))
      return 0 // verification failed

    // if a form was submitted, handle it seperately
    if(href_list["form"])
      HandleForm(href_list)

    return 1 // verification succeeded

  proc
    isValidViewer(client/C)
      // the 'who' check: ensures the link-sender is the form's owner
      return src.owner == C

    UpdatePage(bodyText, styleText, jsText)
      src.body = {"
        <html>
        <head>
          <title> [src.window_name] </title>
          <style type="text/css"> [styleText] </style>
          <script language="javascript"> [jsText] </script>
        </head>
        <body> [bodyText] </body>
        </html>
      "}

    GeneratePage(list/errors=list())
      // Generate the page text and layout by calling UpdatePage() with the body
      // To be implemented in children types

    DisplayPage(list/errors)
      // first generate the page text, then display the page to the owner
      GeneratePage(errors)
      owner << browse(body, "window=[src.window_name]&[window_params]")

    // form functions

    HandleForm(list/href_list)
      // organize the data, and then process the form
      var/formname = href_list["form"]
      var/list/data = href_list.Copy(href_list.Find("form") + 1)
      var/errors = ProcessForm(formname, data) // process data
      if(istype(errors, /list))
        FormSubmitError(formname, errors)
      else
        FormSubmitSuccess(formname)

    ProcessForm(formname, list/params)
      // Processes a single form on the event the submit button was pressed
      var/list/errors = list()
      for(var/item in params) // loop through each name=value item individually
        var/alert = ProcessVariable(formname, item, params[item])
        if(alert) errors[item] = alert
      if(!length(errors)) return null
      return errors

    ProcessVariable(formname, name, value)
      // Called for each control when a form is submitted
      // If applicable, returns a string containing an error message
      // To be implemented in children types

    FormSubmitSuccess(formname)
      // Called when a form is successfully submitted

    FormSubmitError(formname, list/errors)
      // Called when a form has been submitted, but failed verification
      DisplayPage(errors)

